import paho.mqtt.client as mqtt
import time
import sys
from collections import deque
import pickle
import datetime
import uuid
import class_data_encrypt
import class_ia_configure

class init():

    def __init__(self, devNum):

        self.iaCfgFilePath = './ia.ini'
        self.clsCFG = class_ia_configure.init(self.iaCfgFilePath)
        self.clsENCRYPT = class_data_encrypt.init('SeaDeep2021@WSX')
        self.hostIP = self.clsCFG.parseCFGData()['mqtt_broker']
        self.portID = int(self.clsCFG.parseCFGData()['mqtt_port'])
        self.mqttSessionTime = int(self.clsCFG.parseCFGData()['mqtt_session'])
        self.iaHBInterval = int(self.clsCFG.parseCFGData()['ia_hb_interval'])
        self.mqttSeaDeepPubTopic = self.clsCFG.parseCFGData()['sea_deep_mqtt_pub_topic']
        self.qosVal = 1
        self.iaDeviceID = 'IA-' + str(uuid.UUID(int = uuid.getnode()).hex[-12:])
        self.iaNewID = str(uuid.UUID(int = uuid.getnode()).hex[-12:]) + "-" + devNum

    # purpose: mqtt subscribe and parsing the message
    def mqtt_subscribe(self):

        clsCFG = class_ia_configure.init(self.iaCfgFilePath)
        subTopicLists = clsCFG.parseCFGData()['ia_uuid'].split(',')
        print('all ia topics=====>', subTopicLists)

        def on_connect(client, userdata, flags, rc):
            print('ia subscribe connection:', str(rc))
            print('ia subscribe from:', self.iaNewID, 'QOS=', self.qosVal)
            self.clsCFG.msg_log('ia mqtt subscribe: %s ...done.'%(self.iaNewID))
            self.clsCFG.updateCfgCurrentStatus(self.iaNewID)
            
            '''
            for subTopic in subTopicLists:
                print('ia subscribe from:', subTopic, 'QOS=', self.qosVal)
                self.clsCFG.msg_log('ia mqtt subscribe: %s ...done.'%(subTopic))
                self.clsCFG.updateCfgCurrentStatus(subTopic)
            '''

        def on_disconnect(client, userdata, rc):
            logging.info("mqtt disconnected with code: [%s]"%(str(rc)))
            client.connected_flag=False
            client.disconnect_flag=True

        def on_message(client, userdata, msg):

            payloadTopic = msg.topic
            payloadMsg = msg.payload
            print('payloadTopic=======>', payloadTopic)
            print('payloadMsg=======>', payloadMsg)
            #payloadMsgDecode = self.clsENCRYPT.dataDecode(payloadMsg)
            #print('message decoded=======>', payloadMsgDecode)
            #msg_processing(payloadMsgDecode)
            msg_processing(payloadMsg)

        def msg_processing(payloadMsg):
            print('%s-received msg:[%s]'%(datetime.datetime.now().strftime("%Y%m%d-%H%M%S.%f"), payloadMsg))
            self.clsCFG.msg_log(payloadMsg)
            self.clsCFG.msg_queue(payloadMsg)

        client = mqtt.Client(client_id='SERVER', clean_session=True)
        client.reinitialise()
        client.on_connect = on_connect
        client.on_disconnect = on_disconnect
        client.on_message = on_message
        client.connect(self.hostIP, self.portID, self.mqttSessionTime)
       
        client.subscribe(self.iaNewID)
        print('MQTT subscribed host:', self.hostIP, 'port:', self.portID, 'sub_topic:', self.iaNewID)
        client.loop_forever()

    # purpose: mqtt message publish
    def mqtt_hb_publish(self):

        pubTopic = self.mqttSeaDeepPubTopic
        pubMsg = self.clsCFG.getCurrentStatus(self.iaNewID)
        encodedPubMsg = self.clsENCRYPT.dataEncode(pubMsg)
        hbInterval = self.iaHBInterval

        def on_connect(client, userdata, flags, rc):
            print('hb connection:', str(rc))
            print('hb publish to:', pubTopic, 'QOS=', self.qosVal)
            self.clsCFG.updateCfgCurrentStatus(self.iaNewID)

        client = mqtt.Client(client_id='SERVER', clean_session=True)
        client.reinitialise()
        client.on_connect = on_connect
        client.connect(self.hostIP, self.portID, self.mqttSessionTime)
        client.loop_start()
        print('MQTT heartbeat:', self.hostIP, 'port:', self.portID, 'pub_topic:', pubTopic, 'pub_status:', pubMsg)
        client.publish(pubTopic, encodedPubMsg, retain=False, qos=1)
        self.clsCFG.msg_log('[heartbeat]:IA [%s] published to [%s] with [%s]'%(self.iaNewID, pubTopic, pubMsg))
        client.loop_stop()        

        return hbInterval

    # purpose: mqtt message publish
    def mqtt_launch_publish(self):

        pubTopic = self.iaDeviceID
        pubMsg = 'ia_launch'
        hbInterval = self.iaHBInterval

        def on_connect(client, userdata, flags, rc):
            print('ia launch connection:', str(rc))
            print('ia launch publish to:', pubTopic, 'QOS=', self.qosVal)
            self.clsCFG.updateCfgCurrentStatus(self.iaNewID)

        client = mqtt.Client(client_id='SERVER', clean_session=True)
        client.reinitialise()
        client.on_connect = on_connect
        client.connect(self.hostIP, self.portID, self.mqttSessionTime)
        client.loop_start()
        print('MQTT heartbeat launch:', self.hostIP, 'port:', self.portID, 'pub_topic:', pubTopic, 'pub_status:', pubMsg)
        client.publish(pubTopic, pubMsg, retain=False, qos=1)
        client.loop_stop()        

        return hbInterval

    # purpose: mqtt message publish
    def mqtt_publish(self, hostIP, portID, pubTopic, pubMsg, qosVal):

        def on_connect(client, userdata, flags, rc):
            print('connection result:', str(rc))
            print('publish to:', pubTopic, 'QOS=', qosVal)

        def on_disconnect(client, userdata, rc):
            logging.info("mqtt disconnected with code: [%s]"%(str(rc)))
            client.connected_flag=False
            client.disconnect_flag=True

        client = mqtt.Client(client_id='SERVER', clean_session=True)
        client.reinitialise()
        client.on_connect = on_connect
        client.on_disconnect = on_disconnect
        client.connect(hostIP, portID, self.mqttSessionTime)
        client.loop_start()
        print('MQTT free published to:', hostIP, 'port_id:', portID, 'pub_topic:', pubTopic, 'pub_message', pubMsg)
        client.publish(pubTopic, pubMsg, retain=False, qos=1)
        self.clsCFG.msg_log('[free published]:IA [%s] published message to [%s] with [%s]'%(self.iaNewID, pubTopic, pubMsg))
        client.loop_stop()        
